
#	INITIALIZATIONS

#	Control commands in hex
set START	0x12
set RESET	0x11
set IDENTIFY	0x30
set STOP	0x00
set ERROR	0x01
set C104ERROR	0x02
set RECOVER_ERR	0x10
set CPEEK	0x32
set CPOKE	0x66
set PEEK16	0x72
set PEEK32	0x74
set POKE16	0x64
set POKE32	0x6a
set BOOT16	0x44
set BOOT32	0x4a
set BOOTDATA	0x2C
set RUN16	0x54
set RUN32	0x5a
set REBOOT	0x20

set HANDSHAKE		0x80
set SUCCESS_STATUS	0x00


#	Global variables
set ControlLink   	l2
set DataLink		l3

set GenerateFastConfig 0
set ConfigFileName	fastconfig1.tcl
if { $GenerateFastConfig == 1 } {
	set FastConfigFile [ open $ConfigFileName "w" ]
}


# Send a control message and wait for handshake
#	Parameters :
#		Command : Single byte hex command
#		Msg	: Control message with components separated by '.'
#		OutHeader : Destination virtual channel header in hex
#		InHeader  : Source virtual channel header in hex
proc SendControlMsg { Command Msg OutHeader InHeader } {
	global ControlLink 
	global CPOKE POKE32 RUN32
	global HANDSHAKE SUCCESS_STATUS
	global GenerateFastConfig FastConfigFile

	if { $GenerateFastConfig == 1 } {
		puts $FastConfigFile "vcpsend $ControlLink.$OutHeader $InHeader -data $Command.$Msg"
		puts $FastConfigFile "vcprecv $ControlLink.$InHeader $OutHeader -data MsgReceived"
		
	} else {
		vcpsend $ControlLink.$OutHeader $InHeader -data $Command.$Msg -timeout 1000 { puts stderr "SendControlMsg timed out while sending message" }
		vcprecv $ControlLink.$InHeader $OutHeader -data MsgReceived -timeout 1000 { puts stderr "SendControlMsg timed out waiting for ack" }
		set ack [ expr {$Command | $HANDSHAKE }]
		set ack_recvd [ expr [string range $MsgReceived 0 3] ]
		if { $ack_recvd != $ack } {
			puts stderr  "SendControlMsg: Error: Expected $ack, received $MsgReceived"
			exit
		}
		if { ( $Command == $CPOKE)||( $Command == $POKE32 )|| \
		     ( $Command == $RUN32 ) } {
			if { [ string length $MsgReceived ] != 9 } {
				puts stderr "SendControlMsg: Error: Extra bytes received: $MsgReceived"
				exit
			} else {
				if { [ string range $MsgReceived 5 8 ] != $SUCCESS_STATUS } {
					puts stderr "SendControlMsg: Error: Command unsuccessful :Handshake was $MsgReceived"
					exit
				}
			}
		} else	{
			if { [ string length $MsgReceived ] != 4 } {
				puts stderr "SendControlMsg: Error: Extra bytes received: $MsgReceived"
				exit
			}
		}
	}
	return 
}


# Send Start command for the given pair of headers
# Assumes that the headers are 16 bit.
proc SendStartCommand { OutHeader InHeader } {
	global START
	set ReturnHeader [format 0x%04xs $InHeader]
	SendControlMsg $START $ReturnHeader $OutHeader $InHeader
}

# Send a CPoke command to the given pair of headers
#	Parameters :
#		Address : 2 byte address in hex
#		Data	: 4 byte hex data in hex
proc SendCPokeCommand { Address Data OutHeader InHeader } {
	global CPOKE
	set Msg ""
	append Msg $Address "s" "." $Data "l"
	SendControlMsg $CPOKE $Msg $OutHeader $InHeader
}

# Send boot code/data to the given pair of headers
#	Parameters :
#		BootAddress : Starting address in hex
#		BootCode    : List of 32 bit data in hex
proc SendBootCode { BootAddress BootCode OutHeader InHeader } {
	global POKE32
	set BootLength [ llength $BootCode ]
	for {set i 0} {$i < [expr $BootLength]} {set i [expr $i+1]} {
		set PokeAddress $BootAddress 
		append PokeAddress "l"
		SendControlMsg $POKE32 $PokeAddress.[lindex $BootCode $i]l $OutHeader $InHeader
		set BootAddress [format 0x%08x [expr $BootAddress+4]]
	}
}

#	Configure C100
set C100_Default(LinkStartWord)		0x02
proc ConfigureC100 { C100_Parameters } {
	upvar 1 $C100_Parameters C100
	global C100_Default

	set C100_START_CONTROL_LINK_DOWN 	0xfe02

	set C100Header	[expr 0x0ccc]

	set ParametersSpecified [ lsort [ array names C100 ]]
	set DefaultParameters [ lsort [ array names C100_Default ]]
	set j 0
	for {set i 0} {$i < [llength $DefaultParameters]} {set i [expr $i+1]} {
		if { $j < [llength $ParametersSpecified] } {
			if { [lindex $DefaultParameters $i] == [lindex $ParametersSpecified $j] } {
				set j [expr $j + 1]
			}
		} else {
			set C100([lindex $DefaultParameters $i]) $C100_Default([lindex $DefaultParameters $i])
		}
	}


	set InHeader $C100Header
	set OutHeader $C100Header
	SendStartCommand $OutHeader $InHeader
	SendCPokeCommand $C100_START_CONTROL_LINK_DOWN $C100(LinkStartWord) $OutHeader $InHeader
}


#	Configure  T9000

#	Constants 
set MinVirtualChannel 0x80000040
set BytesPerVLCB	0x32
set MaxPacketSize	0x32


#T9000 Control system addresses

#	Default values to be written to T9000 configuration registers
set H1_Default(PLL_ConfigurationWord)	0x0a
set H1_Default(LinkStartWord)		0x02
set H1_Default(LinkConfigWord)		0x14
set H1_Default(BankBaseAddressList)	"0xffffffff 0x80000000 0x80400000 0x80000000"
set H1_Default(BankMaskList)		"0xffffffff 0xffc00000 0xffc00000 0x00000000"
set H1_Default(BankTimingControlList)	"0xfffffe00 0x10884400 0x10884400 0x00080400"
set H1_Default(BankRasBitList)		"0x00000000 0x003ff000 0x003ff000 0x00000000"
set H1_Default(BankFormatControlList)	"0x00000000 0xa000000c 0xa000000c 0x0000000c"
set H1_Default(BankRasStrobeList)	"0xfff00000 0x008c0000 0x008c0000 0xfffc0000"
set H1_Default(BankCasStrobeList)	"0xfff00000 0x087c0000 0x087c0000 0xfffc0000"
set H1_Default(BankProgramStrobeList)	"0xfff00000 0xfffc0000 0xfffc0000 0xfffc0000"
set H1_Default(BankWriteStrobeList)	"0xfff00000 0x00780000 0x00780000 0x00780000"

set H1_Default(RefreshControlWord)	0x4b082600

set H1_Default(VCP_HeaderOffset)	0x00000000
set H1_Default(VCP_LowerLimit)		0x00000000
set H1_Default(VCP_UpperLimit)		0x0000ffff
set H1_Default(VCP_TxMode)		0x00000000

set H1_Default(StartPmiWord)		0x00000000

set H1_Default(BootCode)	 "0xf224f224 0x63f06040 0x24f224f0 0xf06041f2 0xf224f063 	 \
0x24f16040 0xf16041f2 0xf224d040 0x214522d1 0xd340d2fb 0xd540d440 0xd740d640 0xd940d840 0xdb40da40 \
0x60434510 0x40b322fc 0x2067f425 0x20202020 0x21d14024 0xfe2b4f20 0x2443f471 0x4870d0f0 0x71f8f423 \
0x70d5f225 0x40d371d2 0x4a2821d1 0xfa25fb21 0xd6d4fa25 0x7443d930 0x7375d4fa 0x3074f661 0xf0244021 \
0x244f3074 0xf86173f6 0x30745174 0x40202f2f 0x2448f624 0xf56173f0 0xf7617341 0x7341f063 0x61f22544 \
0x74f063f7 0xd4f22548 0xf2254873 0x402275d3 0x11d5f225 0xf1224624 0x44202842 0xd244ff2b 0x7141d140 \
0x3176f124 0xfa25f624 0x202171af 0x2028f840 0xf2254220 0xff2bf042 0x224f2111 0x253276f1 0xc0fa25fa \
0xf00c2aa2 0x28c1fa25 0x202241a4 0xff2b4120 0x20224821 0xff2b4320 0x20202467 0x40212020 0x44202022 \
0x2241ff2b 0x2b452020 0x224921ff 0x2b432020 0x202467ff 0x25202020 0x20202240 0x41ff2b44 0x45202022 \
0x4e27ff2b 0x43202022 0x2067ff2b 0x26202020 0x20224021 0xff2b4420 0x20202241 0x27ff2b45 0x2020224f \
0x67ff2b43 0x20202020 0x22402526 0x2b442020 0x202241ff 0xff2b4520 0x20202241 0x22ff2b42 0xfa25f002 \
0x41ac21c2 0x41202022 0x2241ff2b 0x2b422020 0x202243ff 0xff2b4120 0x20202241 0x67ff2b42 0x20202020 \
0xd1402420 0x20287179 0x63fc6144 0x2cf063f0 0x2540d2f0 0x72d379f4 0xd5f224d4 0x71d6f224 0x254471d7 \
0x4478d8f2 0x70d9f225 0x73daf441 0x7bdbf274 0xf2442024 0x26217cdc 0x2024f24c 0x12def240 0x244c227e \
0xfb2145fa 0x71fc237e 0x202020f6"

set H1_Default(NetworkLoaderAddress)	0x807fe000 
set H1_Default(CacheEnableType)		0x01


#	Configure a T9000 and send the primary boot code
#	Parameters :
#		LinksToBeConfigured : Least significant nibble gives the links
#			that are to be configured for this T9000. Bit 
#			corresponding to link number is set to 1 if link is
#			to be configured, 0 otherwise.
#		VirtualChanInfo	: Information about the virtual channels used
#			for booting this T9000. For each virtual channel two
#			32 bit hex numbers are specified. The first word
#			has the format <HeaderOffsets:16><HeaderLen:8><Link:8>
#			The second word contains header value
proc ConfigureT9000 { LinksToBeConfigured VirtualChanInfo H1_Parameters OutHeader InHeader } {
global START RUN32 POKE32 CPOKE

set H1_PLL_REGISTER_ADDRESS	0x1005
set H1_START_CONTROL_LINK_DOWN	0xfe02
set H1_CONFIGURE_DATA_LINK	"0x8001 0x8101 0x8201 0x8301"

set H1_BANK_BASE_ADDRESS		"0x0202 0x0205 0x0208 0x020b"
set H1_BANK_MASK_ADDRESS		"0x0203 0x0206 0x0209 0x020c"
set H1_BANK_TIMING_CONTROL_ADDRESS	"0x0411 0x0412 0x0413 0x0414"
set H1_BANK_RAS_BIT_ADDRESS		"0x020e 0x020f 0x0210 0x0211"
set H1_BANK_FORMAT_CONTROL_ADDRESS	"0x0204 0x0207 0x020a 0x020d"
set H1_BANK_RAS_STROBE_ADDRESS		"0x0401 0x0402 0x0403 0x0404"
set H1_BANK_CAS_STROBE_ADDRESS		"0x0405 0x0406 0x0407 0x0408"
set H1_BANK_PROGRAM_STROBE_ADDRESS	"0x0409 0x040a 0x040b 0x040c"
set H1_BANK_WRITE_STROBE_ADDRESS	"0x040d 0x040e 0x040f 0x0410"

set H1_PMI_REFRESH_CONTROL_ADDRESS	0x0415

set H1_MEMSTART_ADDRESS			0x410e
set H1_MAX_VCHAN_ADDRESS		0x010f
set H1_HEADER_BASE_ADDRESS		0x0901
set H1_RC_BASE_ADDRESS			0x4110

set H1_VCP_LINK_HEADER_OFFSET_ADDRESS		"0x0808 0x080c 0x0810 0x0814"
set H1_VCP_LINK_HEADER_LOWER_LIMIT_ADDRESS 	"0x0807 0x080b 0x080f 0x0813"
set H1_VCP_LINK_HEADER_UPPER_LIMIT_ADDRESS 	"0x0806 0x080a 0x080e 0x0812"
set H1_VCP_LINK_TXMODE_ADDRESS			"0x0805 0x0809 0x080d 0x0811"

set H1_START_PMI_ADDRESS		0x0212

upvar 1  $H1_Parameters H1
global MinVirtualChannel BytesPerVLCB MaxPacketSize
global H1_Default


set ParametersSpecified [ lsort [ array names H1 ]]
set DefaultParameters [ lsort [ array names H1_Default ]]
set j 0
for {set i 0} {$i < [llength $DefaultParameters]} {set i [expr $i+1]} {
	if { $j < [llength $ParametersSpecified] } {
		if { [lindex $DefaultParameters $i] == [lindex $ParametersSpecified $j] } {
			set j [expr $j + 1]
		}
	} else {
		set H1([lindex $DefaultParameters $i]) $H1_Default([lindex $DefaultParameters $i])
	}
}

SendStartCommand $OutHeader $InHeader
SendCPokeCommand $H1_PLL_REGISTER_ADDRESS $H1(PLL_ConfigurationWord) $OutHeader $InHeader
SendCPokeCommand $H1_START_CONTROL_LINK_DOWN $H1(LinkStartWord) $OutHeader $InHeader

for {set i 0} {$i < 4} {set i [expr $i+1]} {
	if { $LinksToBeConfigured & ( 0x1 << $i ) } { 
		SendCPokeCommand [lindex $H1_CONFIGURE_DATA_LINK $i]	\
 			$H1(LinkConfigWord) $OutHeader $InHeader
	}
}

for {set i 0} {$i < 4} {set i [expr $i+1]} {
	SendCPokeCommand [lindex $H1_BANK_BASE_ADDRESS $i] \
		[lindex $H1(BankBaseAddressList) $i ] $OutHeader $InHeader

	SendCPokeCommand [lindex $H1_BANK_MASK_ADDRESS $i] \
		 [lindex $H1(BankMaskList) $i ] $OutHeader $InHeader

	SendCPokeCommand [lindex $H1_BANK_TIMING_CONTROL_ADDRESS $i] \
 		[lindex $H1(BankTimingControlList) $i ] $OutHeader $InHeader

	SendCPokeCommand [lindex $H1_BANK_RAS_BIT_ADDRESS $i] \
		 [lindex $H1(BankRasBitList) $i ] $OutHeader $InHeader

	SendCPokeCommand [lindex $H1_BANK_FORMAT_CONTROL_ADDRESS $i] \
		 [lindex $H1(BankFormatControlList) $i ] $OutHeader $InHeader

	SendCPokeCommand [lindex $H1_BANK_RAS_STROBE_ADDRESS $i] \
		 [lindex $H1(BankRasStrobeList) $i ] $OutHeader $InHeader

	SendCPokeCommand [lindex $H1_BANK_CAS_STROBE_ADDRESS $i] \
		[lindex $H1(BankCasStrobeList) $i ] $OutHeader $InHeader

	SendCPokeCommand [lindex $H1_BANK_PROGRAM_STROBE_ADDRESS $i] \
		 [lindex $H1(BankProgramStrobeList) $i ] $OutHeader $InHeader

	SendCPokeCommand [lindex $H1_BANK_WRITE_STROBE_ADDRESS $i] \
		 [lindex $H1(BankWriteStrobeList) $i ] $OutHeader $InHeader
}


SendCPokeCommand $H1_PMI_REFRESH_CONTROL_ADDRESS $H1(RefreshControlWord) $OutHeader $InHeader

set NumVirtualChannels	[expr [llength $VirtualChanInfo] / 2 ]
set MemStart [format 0x%08x [expr $MinVirtualChannel + $NumVirtualChannels * ( $BytesPerVLCB + $MaxPacketSize ) ]]
set MaxVChan [format 0x%08x [expr $MinVirtualChannel + $NumVirtualChannels * 8 ]]
set HeaderBase          $MemStart
set RCBase              $MemStart


	SendCPokeCommand $H1_MEMSTART_ADDRESS $MemStart $OutHeader $InHeader
	SendCPokeCommand $H1_MAX_VCHAN_ADDRESS $MaxVChan $OutHeader $InHeader
	SendCPokeCommand $H1_HEADER_BASE_ADDRESS $HeaderBase $OutHeader $InHeader
	SendCPokeCommand $H1_RC_BASE_ADDRESS $RCBase $OutHeader $InHeader

for {set i 0} {$i < 4} {set i [expr $i+1]} {
        SendCPokeCommand [lindex $H1_VCP_LINK_HEADER_OFFSET_ADDRESS $i] \
		 $H1(VCP_HeaderOffset) $OutHeader $InHeader
        SendCPokeCommand [lindex $H1_VCP_LINK_HEADER_LOWER_LIMIT_ADDRESS $i] \
		 $H1(VCP_LowerLimit) $OutHeader $InHeader
        SendCPokeCommand [lindex $H1_VCP_LINK_HEADER_UPPER_LIMIT_ADDRESS $i] \
		 $H1(VCP_UpperLimit) $OutHeader $InHeader
        SendCPokeCommand [lindex $H1_VCP_LINK_TXMODE_ADDRESS $i]	\
		 $H1(VCP_TxMode) $OutHeader $InHeader
}


SendCPokeCommand $H1_START_PMI_ADDRESS $H1(StartPmiWord) $OutHeader $InHeader


SendBootCode $MemStart $H1(BootCode) $OutHeader $InHeader

set BootData "$H1(NetworkLoaderAddress) $LinksToBeConfigured $H1(CacheEnableType) $VirtualChanInfo"
SendBootCode [ format 0x%x [expr $MemStart + [llength $H1(BootCode) ] * 4 ]]\
	 $BootData $OutHeader $InHeader



set BootLength [expr [llength $H1(BootCode)]* 4 + [llength $BootData]*4]
set WS [format 0x%08x [ expr $MemStart + $BootLength]]l
set Entry  ""
append Entry $MemStart "l"
SendControlMsg $RUN32 $WS.$Entry $OutHeader $InHeader

}


# 	C104 Control system addresses

set C104_Default(PLL_ConfigurationWord)	0x0a
set C104_Default(LinkStartWord)		0x02
set C104_Default(LinkConfigWord)	0x14

set C104_Default(PacketMode)		0x14
set C104_Default(RandomBase)		0x00
set C104_Default(RandomRange)		0x00
set C104_Default(RandomSeed)		0x00
set C104_Default(GroupId)		0x00

set C104_Default(LinkIntervals)	"0x00000020 0x00040040 0x00080060 0x000c0080 0x001000a0 \
0x001400c0 0x001800e0 0x001c0100 0x00020100 0x00020100 0x00020100 0x00020100 0x00020100 \
0x00020100 0x00020100 0x00020100 0x00020100 0x00020100 0x00020100 0x00020100 0x00020100 \
0x00020100 0x00020100 0x00020100 0x00020100 0x00020100 0x00020100 0x00020100 0x00020100 \
0x00020100 0x00020100 0x00020100 0x00020100 0x00020100 0x00020100 0x00020000"

# Configure C104 
proc ConfigureC104 { C104_Parameters OutHeader InHeader } {
global START RUN32 POKE32 CPOKE
global C104_Default

set C104_PLL_REGISTER_ADDRESS		0x1005
set C104_CONFIGURE_CONTROL_LINK_UP	0xfd01
set C104_CONFIGURE_DATA_LINK		"0x8001 0x8101 0x8201 0x8301 0x8401 0x8501 0x8601 0x8701"
set C104_DATA_LINK_PACKET_MODE_ADDRESS  "0x8080 0x8180 0x8280 0x8380 0x8480 0x8580 0x8680 0x8780"
set C104_DATA_LINK_RANDOM_BASE_ADDRESS  "0x80a5 0x81a5 0x82a5 0x83a5 0x84a5 0x85a5 0x86a5 0x87a5"
set C104_DATA_LINK_RANDOM_RANGE_ADDRESS "0x80a6 0x81a6 0x82a6 0x83a6 0x84a6 0x85a6 0x86a6 0x87a6"
set C104_DATA_LINK_RANDOM_SEED_ADDRESS  "0x80a7 0x81a7 0x82a7 0x83a7 0x84a7 0x85a7 0x86a7 0x87a7"
set C104_DO_SET_UP_GROUPS		0x1003
set C104_START_DATA_LINK_ADDRESS  	"0x8002 0x8102 0x8202 0x8302 0x8402 0x8502 0x8602 0x8702"

upvar 1 $C104_Parameters C104



set ParametersSpecified [ lsort [ array names C104 ]]
set DefaultParameters [ lsort [ array names C104_Default ]]
set j 0
for {set i 0} {$i < [llength $DefaultParameters]} {set i [expr $i+1]} {
	if { $j < [llength $ParametersSpecified] } {
		if { [lindex $DefaultParameters $i] == [lindex $ParametersSpecified $j] } {
			set j [expr $j + 1]
		}
	} else {
		set C104([lindex $DefaultParameters $i]) $C104_Default([lindex $DefaultParameters $i])
	}
}

	SendStartCommand $OutHeader $InHeader
	SendCPokeCommand $C104_PLL_REGISTER_ADDRESS $C104(PLL_ConfigurationWord) $OutHeader $InHeader
	SendCPokeCommand $C104_CONFIGURE_CONTROL_LINK_UP $C104(LinkConfigWord) $OutHeader $InHeader

	for {set i 0} {$i < 8} {set i [expr $i+1]} {
		set ConfigAddr [lindex $C104_CONFIGURE_DATA_LINK $i]
		SendCPokeCommand $ConfigAddr $C104(LinkConfigWord) $OutHeader $InHeader
	}

	set RandomSeed $C104(RandomSeed)
	for {set i 0} {$i < 8} {set i [expr $i+1]} {
		SendCPokeCommand [lindex $C104_DATA_LINK_PACKET_MODE_ADDRESS $i]\
			 $C104(PacketMode) $OutHeader $InHeader
		SendCPokeCommand [lindex $C104_DATA_LINK_RANDOM_BASE_ADDRESS $i]\
			 $C104(RandomBase) $OutHeader $InHeader
		SendCPokeCommand [lindex $C104_DATA_LINK_RANDOM_RANGE_ADDRESS $i]\
			 $C104(RandomRange) $OutHeader $InHeader
		SendCPokeCommand [lindex $C104_DATA_LINK_RANDOM_SEED_ADDRESS $i]\
			 $RandomSeed $OutHeader $InHeader
		set RandomSeed [format 0x%08x [expr $RandomSeed + 1]]
	}

	for {set i 0} {$i < 36} {set i [expr $i+1]} {
		SendCPokeCommand 0xff[format %02x [expr 128+$i+1]]	\
		 [lindex $C104(LinkIntervals) $i ] $OutHeader $InHeader
	}

	SendCPokeCommand $C104_DO_SET_UP_GROUPS $C104(GroupId) $OutHeader $InHeader
	for {set i 0} {$i < 8} {set i [expr $i+1]} {
		SendCPokeCommand [lindex $C104_START_DATA_LINK_ADDRESS $i ] \
			 $C104(LinkStartWord) $OutHeader $InHeader
	}
}


# Reset 
run #localwrite 0 1
run #localwrite 0 0
run #localwrite 4 0

#	Initialize VCP
run #vcpinit $ControlLink

if { $GenerateFastConfig == 1 } {
	puts $FastConfigFile "run #localwrite 0 1"
	puts $FastConfigFile "run #localwrite 0 0"
	puts $FastConfigFile "run #localwrite 4 0"
	puts $FastConfigFile "set ControlLink  $ControlLink"  
	puts $FastConfigFile "set DataLink $DataLink"
	puts $FastConfigFile "run #vcpinit $ControlLink"
	puts $FastConfigFile "puts stderr \"Configuring C100\""
} else {
	puts stderr "Configuring C100"
}
ConfigureC100 C100_Default
set OutHeader 0[ expr 0x00]
set InHeader  [expr 0x8000]
for {set T9Num 0} {$T9Num < 1} {set T9Num [expr $T9Num+1]} {
set VirtualChanInfoList {"0x00000100 0x00000000 0x00000100 0x00000001 0x00000102 0x00000000"\
			"0x00000100 0x00000001 0x00000102 0x00000000"\
			"0x00000100 0x00000001 0x00000102 0x00000000"\
			"0x00000100 0x00000001" }

	if { $GenerateFastConfig == 1 } {
		puts $FastConfigFile "puts stderr \"Configuring T9000: $T9Num\""
	} else {
		puts stderr "Configuring T9000: $T9Num"
	}
	if { $T9Num == 3 } { 
		# Configure links 0, 1 and 3
		set LinksToBeConfigured	0xb
	} else {
		# Configure links 0, 1, 2 and 3
		set LinksToBeConfigured	0xf
	}
	set VirtualChanInfo [lindex $VirtualChanInfoList $T9Num]
	ConfigureT9000 $LinksToBeConfigured $VirtualChanInfo H1_Default $OutHeader $InHeader
	set InHeader [expr $InHeader + 1]
	set OutHeader 0[expr $OutHeader + 1]
}
#puts stderr "Configuring C104"
#ConfigureC104 C104_Default $OutHeader $InHeader
if { $GenerateFastConfig == 1} { 
	close $FastConfigFile 
}

