

/*{{{  include files*/

#ifdef INMOSC
#include <channel.h>
#include <process.h>
#include <semaphor.h>
#else
#include <conc.h>
#endif
#include <string.h>
#include <stdlib.h>
#include "vcptype.h"
#include "vcpvar.h"
#include "vcpmac.h"
#include "queue.h"
#include "vcpext.h"
#include "debug.h"

/*}}}*/
/*{{{  externs*/

extern ERROR_TYPE VCP_ERROR( ERROR_TYPE );
/*}}}*/

/*{{{  utility functions*/
/*{{{  ProcSetPri*/
/*
 *------------------------------------------------------------------------
 *
 *	ProcSetPri :
 *		Sets the priority of the current thread to the specified
 *		priority. 
 *
 *	Return value :
 *		Current priority
 *
 *------------------------------------------------------------------------
 */
PRIORITY_TYPE ProcSetPri( PRIORITY_TYPE priority )
{
	int CurrentPri;
	ProcGetPri( &CurrentPri );
	if ( CurrentPri == priority )
		return( (PRIORITY_TYPE)CurrentPri );
	if ( priority == High )
		ProcToHigh();
	else
		ProcToLow();
	return( (PRIORITY_TYPE)CurrentPri );
}
/*}}}*/

/*{{{  AckPending*/
/*
 *------------------------------------------------------------------------
 * 
 *	AckPending :
 *		Checks if an Acknowledgement is yet to be sent for the
 *	virtual channel, for which a packet has been received.
 *
 *	Return Value :
 *		True if an ACK is pending, False otherwise
 *
 *------------------------------------------------------------------------
 */
Boolean AckPending( VIRTUAL_CHAN_INFO *VChanInfo[MAX_TABLE_SIZE], unsigned char *packet )
{
	unsigned long key;
	VIRTUAL_CHAN_INFO *element;

	key = *( char *)(packet+1);
	element = VChanInfo[key % MAX_TABLE_SIZE];
	while ( element != NULL ) {
		if ( element->ReceiveHeader.HeaderSize == 1 ){
			if ( element->ReceiveHeader.HeaderValue == *( char *)(packet+1) )
				break;
		}
		if ( element->ReceiveHeader.HeaderSize == 2 ){
			if ( element->ReceiveHeader.HeaderValue == GET_SHORT(packet+1) )
				break;
		}
		element = (VIRTUAL_CHAN_INFO *)QNext( element );
	}
	if ( element != NULL )
		return( element->AckNotYetSent );
	return( False );
}
/*}}}*/

/*{{{  RequestPosted*/
/*
 *------------------------------------------------------------------------
 * 
 *	RequestPosted :
 *		Checks if any receive request has been posted for the
 *	specified virtual channel.
 *
 *	Return Value :
 *		True if a request is posted, False otherwise
 *		ReceiveReq is set to the first posted receive request
 *
 *------------------------------------------------------------------------
 */
Boolean RequestPosted( VIRTUAL_CHAN_INFO *VChan, RECEIVE_REQUEST **ReceiveReq )
{
	RECEIVE_REQUEST *tempReceiveReq;

	*ReceiveReq = NULL;
	if (( VChan == NULL )||( VChan->ReceiveRequest == NULL ))
		return( False );
	tempReceiveReq = VChan->ReceiveRequest;
	while (( tempReceiveReq != NULL )&&( BUFFER_FULL( tempReceiveReq )))
		tempReceiveReq = (RECEIVE_REQUEST *)QNext( tempReceiveReq );
	if ( tempReceiveReq != NULL ){
		*ReceiveReq = tempReceiveReq;
		return( True );
	}
	else 
		return( False );
}
/*}}}*/
/*}}}*/

/*{{{  Sender and receiver threads and timer thread*/

/*{{{  SenderThread*/
/*
 *------------------------------------------------------------------------
 * 
 *	SenderThread :
 *		Packetizes and sends messages and sends 
 *	acknowledgement for messages received. This thread is woken up
 *	whenever a send request is queued, or an ACK is to be sent for
 *	a message received. When the ACK for the last packet of a message
 *	is sent, the user thread that received the message is woken up.
 *
 *------------------------------------------------------------------------
 */
static void SenderThread( int FIXME, WAIT_STRUCT *SenderWaitStruct, int LinkNumber, Channel *TimerChannel )
{
	ACK_TO_BE_SENT 	*tempAck;
	VIRTUAL_CHAN_INFO *tempVChan;
	Channel		*SendChannel;
	SEND_REQUEST	*tempSendReq;
	ERROR_TYPE	ErrorStatus;
	int		MsgLen;
	TIMER_REQUEST	*tempTimerReq;
	RECEIVE_REQUEST *tempReceiveReq;

	SendChannel = SendChannels[LinkNumber];
	while ( VCP_Running[LinkNumber] ) {
		/*{{{  sleep if no requests pending*/
		if ( QEmpty( mySendRequestQ ) && QEmpty( myAckToBeSent )){
			SenderWaitStruct->SleepFlag = True;
			ProcSleep( &(SenderWaitStruct->ProcessDescriptor) );
		}
		/*}}}*/
		/*{{{  Send acknowledgements for packets received*/
		while ( !QEmpty( myAckToBeSent )){
			DeQueue( myAckToBeSent, tempAck );
			tempVChan = tempAck->ReceiveRequest->VirtualChanInfo;
			tempVChan->AckNotYetSent = False;
			SEND_ACK( SendChannel, tempVChan->SendHeader, tempAck->TimeOut, ErrorStatus);
			tempReceiveReq = tempAck->ReceiveRequest;
#if 1
			if ( !tempVChan->IgnoreFlag ){
				WAKEUP_RECEIVER_IF_REQUIRED( myVirtualChanInfo, tempReceiveReq );
			}
			else  {
				free( tempReceiveReq );
			}
#else
			WAKEUP_RECEIVER_IF_REQUIRED( myVirtualChanInfo, tempReceiveReq );
#endif
			vcp_free( tempAck );
		}
		/*}}}*/
		/*{{{  Send any queued  packets*/
		while ( !QEmpty( mySendRequestQ )){
			DeQueue( mySendRequestQ, tempSendReq );
			tempVChan = tempSendReq->VirtualChanInfo;
			if ( ACK_DUE(tempVChan) ){
				EnQueue( myPendingSendRequests, tempSendReq );
			}
			else {
				tempVChan->AckExpected.Flag = True;
				tempVChan->AckExpected.SendRequest = tempSendReq;
				if ( tempSendReq->TimeOut > 0 ){
					MEM_ALLOCATE( tempTimerReq, TIMER_REQUEST, ErrorStatus, continue );
					tempTimerReq->DeadLine = ProcTime()+tempSendReq->TimeOut;
					tempTimerReq->IsSendReq = True;
					tempTimerReq->request.SendRequest = tempSendReq;
					tempSendReq->TimerRequest = tempTimerReq;
					ADD_TIMER_Q( myTimerRequestQ, tempTimerReq );
					ChanOutInt( TimerChannel, tempSendReq->TimeOut );
				}
				TRANSMIT_MESSAGE( SendChannel, tempVChan->SendHeader, tempSendReq, ErrorStatus );
			}
		
		}
		/*}}}*/
	}
}
/*}}}*/

/*{{{  ReceiverThread*/
/*
 *------------------------------------------------------------------------
 * 
 *	ReceiverThread :
 *		Receives all messages that arrive on the specified link 
 *	and processes them. 
 *	If an acknowledgement arrives, any pending send request is queued, 
 *	and the sender thread woken up.
 *	If a request has been posted, the message is copied onto the user's
 *	buffer and the sender thread is informed that an acknowledgement
 *	is to be sent.
 *	If no request is posted, the message is queued.
 *
 *------------------------------------------------------------------------
 */
static void ReceiverThread( int FIXME, int LinkNumber, Channel *TimerChannel )
{
	/*{{{  local variables*/
	Channel		*ReceiveChan;
	unsigned char 	*packet;
	int		PacketLength;
	int		MessageLength;
	int		len, HeaderSize;
	VIRTUAL_CHAN_INFO *tempVChan;
	ACK_TO_BE_SENT	*tempAck;
	ERROR_TYPE	ErrorStatus;
	unsigned char 	*RecvBuf, *UserBuf;
	RECEIVE_REQUEST	*tempReceiveReq;
	PACKETS_RECEIVED *tempPacket;
	MESSAGE_LIST	*tempMessageList;
	/*}}}*/

	ReceiveChan = ReceiveChannels[LinkNumber];
	MEM_ALLOCATE( tempPacket, PACKETS_RECEIVED, ErrorStatus, return );
	packet = (unsigned char *)tempPacket->PacketBuffer;
	while ( VCP_Running[LinkNumber] ) {
		ErrorStatus = VCP_OK;
		/*{{{  Receive all messages if required*/
		if ( VCP_ReceiveAllMessages[LinkNumber] == True ) {
			ChanIn( ReceiveChan, packet, MIN_PACKET_SIZE );
			tempMessageList = MEM_ALLOC( MESSAGE_LIST );
			PacketLength = PACKET_LENGTH( packet );
			UserBuf = vcp_malloc( PacketLength + 1 );
			if (( tempMessageList == NULL )||( UserBuf == NULL ))
				VCP_ERROR( OutOfMemory );
			else {
				len = ( PacketLength+1 > MIN_PACKET_SIZE )?MIN_PACKET_SIZE:PacketLength+1;
				CopyMessage( UserBuf, packet, len );
				len = (PacketLength-(MIN_PACKET_SIZE-1));
				if ( len > 0 )
					ChanIn( ReceiveChan, (UserBuf+MIN_PACKET_SIZE), len );
				tempMessageList->MessagePtr = UserBuf;
				ADD_TO_MESSAGE_LIST( ListOfMessages[LinkNumber], tempMessageList );
				if ( ReceiveWaitStruct[LinkNumber].SleepFlag ){
					ReceiveWaitStruct[LinkNumber].SleepFlag = False;
					ProcAwaken( ReceiveWaitStruct[LinkNumber].ProcessDescriptor );
				}
			}
			continue;
		}
		/*}}}*/
		ChanIn( ReceiveChan, packet, MIN_PACKET_SIZE );
		PacketLength = PACKET_LENGTH( packet );
		if ( PacketLength > MAX_PACKET_SIZE ){
			ErrorStatus = VCP_ERROR( PacketTooLong );
		}
		else if ( PacketLength < MIN_MESSAGE_SIZE ) {
			ErrorStatus = VCP_ERROR( PacketTooShort );
			continue;
		}
		else if ( AckPending( VirtualChanInfo[LinkNumber], packet ))
			ErrorStatus = VCP_ERROR( UnExpectedPacket );
		GET_VIRTUAL_CHAN_INFO( VirtualChanInfo[LinkNumber], PACKET_HEADER(packet), tempVChan );
		/*{{{  process acknowledgement*/
		if ( IsACKNOWLEDGEMENT( packet )) {
			if (( tempVChan == NULL )||( !ACK_DUE(tempVChan))){
				ErrorStatus = VCP_ERROR( UnExpectedAck );
			}
			else {
				SEND_REQUEST *tempSendReq;
				tempVChan->AckExpected.Flag = False;
				tempSendReq = tempVChan->AckExpected.SendRequest;
				if ( tempSendReq->TimeOut > 0 ){
					QDelete( myTimerRequestQ, tempSendReq->TimerRequest );
					ChanOutInt( TimerChannel, tempSendReq->TimeOut );
				}
				if ( tempSendReq->BufferSize > 0 ){
					EnQueue( mySendRequestQ, tempSendReq );
					if ( SenderWaitStruct[LinkNumber].SleepFlag == True ){
						SenderWaitStruct[LinkNumber].SleepFlag = False;
						ProcAwaken( SenderWaitStruct[LinkNumber].ProcessDescriptor );
					}
					
				}
				else {
					DELETE_VCHAN( VirtualChanInfo[LinkNumber], tempVChan );
					tempSendReq->Error = VCP_OK;
					ProcAwaken( tempSendReq->SenderProcess );
					if ( !QEmpty( myPendingSendRequests )) {
						DeQueue( myPendingSendRequests, tempSendReq );
						EnQueue( mySendRequestQ, tempSendReq );
						if ( SenderWaitStruct[LinkNumber].SleepFlag == True ){
							SenderWaitStruct[LinkNumber].SleepFlag = False;
							ProcAwaken( SenderWaitStruct[LinkNumber].ProcessDescriptor );
						}
		
					}
				}
			}
		}
		/*}}}*/
		/*{{{  process posted request*/
		else if ( RequestPosted( tempVChan, &tempReceiveReq )){ /* request posted */
			HeaderSize = tempVChan->ReceiveHeader.HeaderSize;
			RecvBuf = packet + (HeaderSize ) + 1;
			PacketLength = ( PacketLength - HeaderSize );
			/* len = length of message already received in packet*/
			len = ( MIN_PACKET_SIZE - (HeaderSize+1) );
			len = ( PacketLength > len )?len:PacketLength;
			MessageLength = tempReceiveReq->MessageLength;
			UserBuf = &(tempReceiveReq->BufferPtr[MessageLength]);
			CopyMessage( UserBuf, RecvBuf, len  );
			tempReceiveReq->MessageLength += len;
			UserBuf += len;
			PacketLength -=  len;
			if ( PacketLength > 0 ) {
				len = PacketLength;
				ChanIn( ReceiveChan, UserBuf, len );
				tempReceiveReq->MessageLength += len;
			}
			tempReceiveReq->EOMFlag = IS_EOM_PACKET( packet );
			SET_AckToBeSent( LinkNumber, tempReceiveReq );
		}
		/*}}}*/
		/*{{{  process unsolicited packet*/
		else {	/* Request not posted */
			tempPacket->Error = ErrorStatus;
			len = (MIN_PACKET_SIZE-1);
			if ( PacketLength > (MIN_PACKET_SIZE-1))
				ChanIn( ReceiveChan, (&(tempPacket->PacketBuffer[len+1])), PacketLength-len);
			GET_VIRTUAL_CHAN_INFO( VirtualChanInfo[LinkNumber], PACKET_HEADER(packet), tempVChan );
#if 1
			if (( tempVChan != NULL )&&( tempVChan->IgnoreFlag )){
				tempReceiveReq = MEM_ALLOC(RECEIVE_REQUEST);
				tempReceiveReq->VirtualChanInfo = tempVChan;
				tempReceiveReq->BufferSize = 0;
				tempReceiveReq->BufferPtr = NULL;
				tempReceiveReq->SleepFlag = False;
				tempReceiveReq->TimeOut = -1;
				SET_AckToBeSent( LinkNumber, tempReceiveReq );
				continue;
			} 
#endif
			if ( RequestPosted( tempVChan, &tempReceiveReq )){ /* request posted now */
				len = PACKET_LENGTH(tempPacket->PacketBuffer) - tempVChan->ReceiveHeader.HeaderSize;
				RecvBuf = &(tempPacket->PacketBuffer[tempVChan->ReceiveHeader.HeaderSize+1]);
				CopyMessage( tempReceiveReq->BufferPtr, RecvBuf, len );
				tempReceiveReq->EOMFlag =  IS_EOM_PACKET(tempPacket->PacketBuffer);
				SET_AckToBeSent( LinkNumber, tempReceiveReq );
			}
			else {
				/* Update ACK NotYetSent -- Size of header ??? */
				AddTable( myPacketsReceived, *(tempPacket->PacketBuffer+1), tempPacket );
				
				MEM_ALLOCATE( tempPacket, PACKETS_RECEIVED, ErrorStatus, return );
				packet = tempPacket->PacketBuffer;
			}
		}
		/*}}}*/

	}
	vcp_free( tempPacket );
}
/*}}}*/

/*{{{  TimerThread*/
/*
 *------------------------------------------------------------------------
 * 
 *	TimerThread :
 *		Checks for timeout while sending or receiving messages.
 *	This thread is woken up by the SenderThread or by any thread
 *	that waits for a message, by sending a message on the 
 *	SenderChannel or ReceiverChannel. On timeout, send/receive
 *	requests are aborted and the sender/receiver thread woken up.
 *
 *------------------------------------------------------------------------
 */
static void TimerThread( int FIXME, int LinkNumber , Channel *SenderChannel, Channel *ReceiverChannel )
{
	TIMER_REQUEST *tempRequest;
	int ChanNum;
	int TimeOut;

	while (1) {
		while ( !QEmpty( myTimerRequestQ )){
			if ( ProcTime() >= myTimerRequestQ->DeadLine ){
				DeQueue( myTimerRequestQ, tempRequest );
				if ( tempRequest->IsSendReq ){
					ABORT_SEND_REQUEST( tempRequest->request.SendRequest )
				}
				else {
					ABORT_RECV_REQUEST( tempRequest->request.ReceiveRequest )
				}
				vcp_free( tempRequest );
			}
			else
				break;
		}
		if ( !QEmpty( myTimerRequestQ )){
			TimeOut = myTimerRequestQ->DeadLine - ProcTime(); 
			ChanNum = ProcTimerAlt( TimeOut,
				SenderChannel, ReceiverChannel, NULL );
		}
		else {
			ChanNum = ProcAlt( SenderChannel, ReceiverChannel, NULL );
		}
		if ( ChanNum == 0 )
			ChanInInt( SenderChannel );
		else if ( ChanNum == 1 ){
			ChanInInt( ReceiverChannel );
		}
	}
}
/*}}}*/

/*}}}*/

/*{{{  VCP functions*/
/*{{{  VCP_Init*/

/*
 *------------------------------------------------------------------------
 *
 *	VCP_Init :
 *		Initialize VCP on the specified link. Initializes all VCP data
 *		structures when invoked for the first time.
 *	Return Value :
 *		VCP_OK if successful, Error Number otherwise
 *
 *------------------------------------------------------------------------
 */
ERROR_TYPE VCP_Init( int LinkNumber )
{
	int i;

	if ( !VCP_Initialized ) {	/* Initialize all data structures */
		VCP_Initialized = True;
		for ( i = 0; i < NUM_LINKS; i++ ) {
			VCP_Running[i] = False;
			SenderProc[i] = ReceiverProc[i] = NULL;
			SenderWaitStruct[i].SleepFlag = False;
			ReceiveWaitStruct[i].SleepFlag = False;
			VCP_ReceiveAllMessages[i] = False;
			/* FIX later: seminit does not work */
			TimerSemaphore[i].value = 1;
			TimerSemaphore[i].sf = NULL;
			TimerSemaphore[i].sb = NULL;
		}
	}
	VCP_Running[LinkNumber] = True;
	InitQ( SendRequestQ[LinkNumber] );
	InitQ( PendingSendRequests[LinkNumber] );
	InitQ( AckToBeSent[LinkNumber] );
	InitQ( TimerRequestQ[LinkNumber] );
	Timer_SenderChannel[LinkNumber] = ChanAlloc();
	Timer_ReceiverChannel[LinkNumber] = ChanAlloc();
	InitTable( VirtualChanInfo[LinkNumber] );
	InitTable( PacketsReceived[LinkNumber] );
	InitQ( ListOfMessages[LinkNumber] );
	SenderProc[LinkNumber] = ProcAlloc( SenderThread, DEFAULT_STACKSIZE, 
		3, &(SenderWaitStruct[LinkNumber]), LinkNumber, Timer_SenderChannel[LinkNumber] );
	ReceiverProc[LinkNumber] = ProcAlloc( ReceiverThread, 
		DEFAULT_STACKSIZE, 2, LinkNumber, Timer_ReceiverChannel[LinkNumber] );
	TimerProc[LinkNumber] = ProcAlloc( TimerThread, DEFAULT_STACKSIZE, 
		3, LinkNumber, Timer_SenderChannel[LinkNumber], Timer_ReceiverChannel[LinkNumber] );
	if (( SenderProc[LinkNumber] == NULL )||
	    ( ReceiverProc[LinkNumber] == NULL )||
	    ( TimerProc[LinkNumber] == NULL )){
		VCP_Running[LinkNumber] = False;
		return( OutOfMemory );
	}
	else {
		ProcRunHigh( SenderProc[LinkNumber] );
		ProcRunHigh( ReceiverProc[LinkNumber] );
		ProcRunHigh( TimerProc[LinkNumber] );
		return( VCP_OK );
	}
}
/*}}}*/

/*{{{  VCP_Quit*/

/*
 *------------------------------------------------------------------------
 *
 *	VCP_Quit :
 *		Terminate VCP on the specified link
 *	Return Value :
 *		VCP_OK if successful, Error Number otherwise 
 *
 *------------------------------------------------------------------------
 */

ERROR_TYPE VCP_Quit( int LinkNumber )
{
	VCP_Running[LinkNumber] = False;
	SenderProc[LinkNumber] = NULL;
	ReceiverProc[LinkNumber] = NULL;
	SenderWaitStruct[LinkNumber].SleepFlag = False;
	ReceiveWaitStruct[LinkNumber].SleepFlag = False;
	return( VCP_OK );
}

/*}}}*/

/*{{{  VCP_ReceiveMessage*/
/*
 *------------------------------------------------------------------------
 *
 *	VCP_ReceiveMessage :
 *		Receive a message on the specified link, for the virtual channel
 *	specified in ReceiveHeader. SendHeader specifies the header to which
 *	ACK is to be sent. buffer is a pointer to the destination buffer, 
 *	*count is the size of the buffer and TimeOut specifies the number of
 *	microseconds to wait for the message
 *
 *	Return Value :
 *		VCP_OK if successful, Error Number otherwise 
 *		*count is set to the actual size of message received
 *		*EOMFlag is set if End Of Message is received
 *
 *------------------------------------------------------------------------
 */

ERROR_TYPE VCP_ReceiveMessage( int LinkNumber , VIRTUAL_CHAN ReceiveHeader, 
	HEADER *SendHeader, char *buffer, int *count, Boolean *EOMFlag,
	int TimeOut )
{
	PRIORITY_TYPE	CurrentPriority;
	PACKETS_RECEIVED *tempPacket = NULL;
	VIRTUAL_CHAN_INFO *tempVChan;
	Boolean		ReqPosted = False;
	ERROR_TYPE	tempError = VCP_OK;
	RECEIVE_REQUEST	*tempReceiveReq;
	int len;


	if (( !VCP_Initialized )||( VCP_Running[LinkNumber] == False )){
		return( VCP_ERROR( VCPNotInitialized ));
	}
	if ( VCP_ReceiveAllMessages[LinkNumber] )
		return( VCP_ERROR( VCP_ReceiveAllSet ));
	CurrentPriority = ProcSetPri( High );
	GET_PACKET_FROM_TABLE( PacketsReceived[LinkNumber], ReceiveHeader, tempPacket );
	ADD_VCHAN( VirtualChanInfo[LinkNumber], SendHeader, ReceiveHeader, tempVChan );
	if ( tempPacket != NULL ) {
		len = PACKET_LENGTH(tempPacket->PacketBuffer) - ReceiveHeader.HeaderSize;
		len = ( len > *count )?*count:len;
		CopyMessage( buffer, tempPacket->PacketBuffer+1+ReceiveHeader.HeaderSize, len );

		POST_REQUEST( buffer, *count, tempVChan, tempReceiveReq, TimeOut  );
		ReqPosted = True;
		SET_AckToBeSent( LinkNumber, tempReceiveReq );
		tempReceiveReq->MessageLength = len;
		tempReceiveReq->EOMFlag = IS_EOM_PACKET(tempPacket->PacketBuffer);
		tempReceiveReq->Error = tempPacket->Error;
		vcp_free( tempPacket );
	}
	else {
		POST_REQUEST( buffer, *count, tempVChan, tempReceiveReq, TimeOut  );
		ReqPosted = True;
	}
	if ( ReqPosted ) {
		tempReceiveReq->SleepFlag = True;
		ProcSleep( &(tempReceiveReq->ReceiverProcess) );
		*count = tempReceiveReq->MessageLength;
		*EOMFlag = tempReceiveReq->EOMFlag;
		tempError = tempReceiveReq->Error;
		vcp_free( tempReceiveReq );
	}
	
	ProcSetPri( CurrentPriority );
	return( tempError );
}
/*}}}*/

/*{{{  VCP_IgnoreMessage*/
/*
 *------------------------------------------------------------------------
 *
 *	VCP_IgnoreMessage :
 *		Ignore a message on the specified link, for the virtual channel
 *	specified in ReceiveHeader. SendHeader specifies the header to which
 *	ACK is to be sent. buffer is a pointer to the destination buffer, 
 *	*count is the size of the buffer and TimeOut specifies the number of
 *	microseconds to wait for the message
 *
 *	Return Value :
 *		VCP_OK if successful, Error Number otherwise 
 *		*count is set to the actual size of message received
 *		*EOMFlag is set if End Of Message is received
 *
 *------------------------------------------------------------------------
 */

ERROR_TYPE VCP_IgnoreMessage( int LinkNumber , VIRTUAL_CHAN ReceiveHeader, 
	HEADER *SendHeader )
{
	PRIORITY_TYPE	CurrentPriority;
	PACKETS_RECEIVED *tempPacket = NULL;
	VIRTUAL_CHAN_INFO *tempVChan;
	Boolean		ReqPosted = False;
	ERROR_TYPE	tempError = VCP_OK;
	RECEIVE_REQUEST	*tempReceiveReq;
	int len;


	if (( !VCP_Initialized )||( VCP_Running[LinkNumber] == False )){
		return( VCP_ERROR( VCPNotInitialized ));
	}
	if ( VCP_ReceiveAllMessages[LinkNumber] )
		return( VCP_ERROR( VCP_ReceiveAllSet ));
	CurrentPriority = ProcSetPri( High );
	GET_PACKET_FROM_TABLE( PacketsReceived[LinkNumber], ReceiveHeader, tempPacket );
	ADD_VCHAN( VirtualChanInfo[LinkNumber], SendHeader, ReceiveHeader, tempVChan );
	if ( tempPacket != NULL ) {

		SET_AckToBeSent( LinkNumber, tempReceiveReq );
		vcp_free( tempPacket );
	}
	tempVChan->IgnoreFlag = True;
	
	ProcSetPri( CurrentPriority );
	return( VCP_OK );
}
/*}}}*/


/*{{{  VCP_ReceiveEverything*/
/*
 *------------------------------------------------------------------------
 *
 *	VCP_ReceiveEverything :
 *		Sets flag to indicate that all messages are to be received.
 *	All incoming messages will be buffered by the receiver thread.
 *
 *	Return Value :
 *		Pointer to MessageList where all messages will be queued.
 *		*Wait is set to the wait structure for the receiver
 *		*RunFlag points to the VCP_Running flag for the specified link
 *
 *------------------------------------------------------------------------
 */
MESSAGE_LIST **VCP_ReceiveEverything( int LinkNumber, WAIT_STRUCT **Wait, 
	Boolean **RunFlag )
{
	if (( !VCP_Initialized )||( VCP_Running[LinkNumber] == False )){
		VCP_ERROR( VCPNotInitialized );
		*Wait = NULL; *RunFlag = NULL;
		return( NULL );
	}
	if ( VCP_ReceiveAllMessages[LinkNumber] ){
		*Wait = NULL; *RunFlag = NULL;
		VCP_ERROR( VCP_ReceiveAllSet );
		return( NULL );
	}
	VCP_ReceiveAllMessages[LinkNumber] = True;
	ReceiveWaitStruct[LinkNumber].SleepFlag = False;
	*Wait = &(ReceiveWaitStruct[LinkNumber] );
	*RunFlag = &(VCP_Running[LinkNumber]);
	return( &(ListOfMessages[LinkNumber]) );
}
/*}}}*/

/*{{{  VCP_SendMessage*/
/*
 *------------------------------------------------------------------------
 *
 *	VCP_SendMessage :
 *		Send a message on the specified link, for the virtual channel
 *	specified in SendHeader. ReceiveHeader specifies the header where ACK
 *	would be received. Buffer is a pointer to the source buffer, and
 *	count is the size of the buffer, EOMFlag indicates if this is the
 *	last packet of a message. TimeOut is the number of microseconds to
 *	wait for the message acknowledgement.
 *
 *	Return Value :
 *		VCP_OK if successful, Error Number otherwise 
 *
 *------------------------------------------------------------------------
 */
ERROR_TYPE VCP_SendMessage( int LinkNumber, HEADER *SendHeader, 
	VIRTUAL_CHAN ReceiveHeader, char *Buffer, int BufferSize, 
	Boolean EOMFlag, int TimeOut )
{
	PRIORITY_TYPE CurrentPriority;
	SEND_REQUEST	*tempSendReq;
	ERROR_TYPE	ret;

	if (( !VCP_Initialized )||( VCP_Running[LinkNumber] == False )){
		return( VCP_ERROR( VCPNotInitialized ));
	}
	CurrentPriority = ProcSetPri( High );
	tempSendReq = MEM_ALLOC( SEND_REQUEST );
	if ( tempSendReq == NULL )
		return( VCP_ERROR( OutOfMemory ));
	else {
		tempSendReq->EOMFlag = EOMFlag;
		tempSendReq->BufferPtr = Buffer;
		tempSendReq->BufferSize = BufferSize;
		tempSendReq->TimeOut = TimeOut ;
		ADD_VCHAN( VirtualChanInfo[LinkNumber], SendHeader, ReceiveHeader, tempSendReq->VirtualChanInfo );
		EnQueue( SendRequestQ[LinkNumber], tempSendReq );
		if ( SenderWaitStruct[LinkNumber].SleepFlag == True ){
			SenderWaitStruct[LinkNumber].SleepFlag = False;
			ProcAwaken( SenderWaitStruct[LinkNumber].ProcessDescriptor );
		}
		ProcSleep( &(tempSendReq->SenderProcess) );
		ret = tempSendReq->Error;
		vcp_free( tempSendReq );
	}

	ProcSetPri( CurrentPriority );
	return( ret );

}
/*}}}*/
/*{{{  VCP_InspectQ*/
/*
 *------------------------------------------------------------------------
 *
 *	VCP_InspectQ :
 *		Return headers of pending messages
 *
 *	Return Value :
 *		String of form "<Header1> <Header2> ..."
 *
 *------------------------------------------------------------------------
 */

char *VCP_InspectQ( int LinkNumber )
{
	PRIORITY_TYPE	CurrentPriority;
	char *HeaderList;
	int NumMessages, i;
	PACKETS_RECEIVED *tempPacketsReceived;
	char tempHeader[6];

	if (( !VCP_Initialized )||( VCP_Running[LinkNumber] == False ))
		return( VCP_ERROR( VCPNotInitialized ));
	CurrentPriority = ProcSetPri( High );
	NumMessages = 0;
	for ( i = 0; i < MAX_TABLE_SIZE; i++ ){
		tempPacketsReceived = PacketsReceived[LinkNumber][i];
		while ( !QEmpty( tempPacketsReceived )){
			NumMessages++;
			tempPacketsReceived = QNext( tempPacketsReceived );
		}
	}
	HeaderList = (char *)vcp_malloc((( NumMessages * 5) + 1) * sizeof(char));
	*HeaderList = '\0';
	for ( i = 0; i < MAX_TABLE_SIZE; i++ ){
		tempPacketsReceived = PacketsReceived[LinkNumber][i];
		while ( !QEmpty( tempPacketsReceived )){
			sprintf( tempHeader, "%02x%02x ", tempPacketsReceived->PacketBuffer[1], tempPacketsReceived->PacketBuffer[2] );
			strcat( HeaderList, tempHeader );
			tempPacketsReceived = QNext( tempPacketsReceived );
		}
	}
	ProcSetPri( CurrentPriority );
	return( HeaderList );
}
/*}}}*/

/*}}}*/

